% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( Matrix )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [succinylcoA_Matrix , GDP_Matrix , Pi_Matrix , succinate_Matrix , GTP_Matrix , COAS_Matrix ]
%
%
% Free Parameters:
% [Vmf_SCS ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
%% LIST OF STATE VARIABLES
% 1 succinylcoA_Matrix
% 2 GDP_Matrix
% 3 Pi_Matrix
% 4 succinate_Matrix
% 5 GTP_Matrix
% 6 COAS_Matrix
% 7 h_Matrix
% 8 m_Matrix
% 9 k_Matrix

% PARTIAL VOLUME FRACTIONS
VWater_Matrix = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_Matrix = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
succinylcoA_Matrix = x(1);
GDP_Matrix = x(2);
Pi_Matrix = x(3);
succinate_Matrix = x(4);
GTP_Matrix = x(5);
COAS_Matrix = x(6);
% Concentrations of H, Mg, and K
h_Matrix = x(7);
m_Matrix = x(8);
k_Matrix = x(9);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% succinylcoA_Matrix
Kh(1) = 0.00010945977234272518557357811719299434116692282259464;
Km(1) = Inf;
Kk(1) = Inf;
% GDP_Matrix
Kh(2) = 4.9861302786366646222352468770244726670171075966209e-07;
Km(2) = 0.0027895056366051803123373709070165205048397183418274;
Kk(2) = 0.038704692519267294703588788706838386133313179016113;
% Pi_Matrix
Kh(3) = 2.1306351186738022140046010417802069270010179025121e-07;
Km(3) = 0.032137949367841568693027198833078728057444095611572;
Kk(3) = 0.37888645618434613204783545370446518063545227050781;
% succinate_Matrix
Kh(4) = 6.3607610505449683980985081666048586157558020204306e-06;
Km(4) = 0.063389622025238692715198851601599017158150672912598;
Kk(4) = 0.44392446615653530228584600081376265734434127807617;
% GTP_Matrix
Kh(5) = 3.689930295417079571699083863356438683922533527948e-07;
Km(5) = 0.00031191504590251855960278404289454101672163233160973;
Kk(5) = 0.019365029468365275999719443689173203893005847930908;
% COAS_Matrix
Kh(6) = 7.5727660576433292701044864528477085219293485351955e-09;
Km(6) = Inf;
Kk(6) = Inf;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_Matrix/Kh(1) + m_Matrix/Km(1) + k_Matrix/Kk(1);
P( 2 ) = 1  + h_Matrix/Kh(2) + m_Matrix/Km(2) + k_Matrix/Kk(2);
P( 3 ) = 1  + h_Matrix/Kh(3) + m_Matrix/Km(3) + k_Matrix/Kk(3);
P( 4 ) = 1  + h_Matrix/Kh(4) + m_Matrix/Km(4) + k_Matrix/Kk(4);
P( 5 ) = 1  + h_Matrix/Kh(5) + m_Matrix/Km(5) + k_Matrix/Kk(5);
P( 6 ) = 1  + h_Matrix/Kh(6) + m_Matrix/Km(6) + k_Matrix/Kk(6);

%% THERMODYNAMIC EQUATIONS
DGro_SCS =47.4869;

Keq_SCS_Matrix = exp(-DGro_SCS/RT)/P(1)/P(2)/P(3)*P(4)*P(5)*P(6)/h_Matrix;

%% FLUX EQUATIONS
%SCS_Matrix
a=GDP_Matrix;
b=succinylcoA_Matrix;
c=Pi_Matrix;
p=COAS_Matrix;
q=succinate_Matrix;
r=GTP_Matrix;
KmA=16e-6;
KmB=55e-6;
KmC=660e-6;
KmP=20e-6;
KmQ=880e-6;
KmR=11.1e-6;
Kia=5.5e-6;
Kib=100e-6;
Kic=2000e-6;
Kip=20e-6;
Kiq=3000e-6;
Kir=11.1e-6;
Vmf_SCS=par(1);
Vmr=Vmf_SCS/Keq_SCS_Matrix*KmP*Kiq*Kir/(Kia*Kib*KmC);
J_SCS_Matrix=(Vmf_SCS*Vmr*a*b*c-Vmf_SCS*Vmr*(p*q*r/Keq_SCS_Matrix))...
/(Vmr*Kia*Kib*KmC+Vmr*Kib*KmC*a+Vmr*Kia*KmB*c...
+Vmr*KmC*a*b+Vmr*KmB*a*c+Vmr*KmA*b*c+Vmr*a*b*c...
+Vmf_SCS*Kir*KmQ*p/Keq_SCS_Matrix+Vmf_SCS*Kiq*KmP*r/Keq_SCS_Matrix...
+Vmf_SCS*KmR*p*q/Keq_SCS_Matrix+Vmf_SCS*KmQ*p*r/Keq_SCS_Matrix...
+Vmf_SCS*KmP*q*r/Keq_SCS_Matrix+Vmf_SCS*p*q*r/Keq_SCS_Matrix...
+Vmf_SCS*KmQ*Kir*a*p/Kia/Keq_SCS_Matrix+Vmr*Kia*KmB*c*r/Kir...
+Vmf_SCS*KmQ*Kir*a*b*p/Kia/Kib/Keq_SCS_Matrix+Vmr*KmA*b*c*r/Kir...
+Vmf_SCS*KmR*a*p*q/Kia/Keq_SCS_Matrix+Vmr*Kia*KmB*c*q*r/Kiq/Kir...
+Vmf_SCS*Kir*KmQ*a*b*c*p/Kia/Kib/Kic/Keq_SCS_Matrix...
+Vmf_SCS*Kip*KmR*a*b*c*q/Kia/Kib/Kic/Keq_SCS_Matrix...
+Vmf_SCS*KmR*a*b*p*q/Kia/Kib/Keq_SCS_Matrix...
+Vmr*KmA*b*c*q*r/Kiq/Kir...
+Vmr*KmA*Kic*b*p*q*r/Kip/Kiq/Kir...
+Vmr*Kia*KmB*c*p*q*r/Kip/Kiq/Kir...
+Vmf_SCS*KmR*a*b*c*p*q/Kia/Kib/Kic/Keq_SCS_Matrix...
+Vmr*KmA*b*c*p*q*r/Kip/Kiq/Kir);

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 1*J_SCS_Matrix ) / VWater_Matrix; % succinylcoA_Matrix
f(2,:) = ( 0  - 1*J_SCS_Matrix ) / VWater_Matrix; % GDP_Matrix
f(3,:) = ( 0  - 1*J_SCS_Matrix ) / VWater_Matrix; % Pi_Matrix
f(4,:) = ( 0  + 1*J_SCS_Matrix ) / VWater_Matrix; % succinate_Matrix
f(5,:) = ( 0  + 1*J_SCS_Matrix ) / VWater_Matrix; % GTP_Matrix
f(6,:) = ( 0  + 1*J_SCS_Matrix ) / VWater_Matrix; % COAS_Matrix

%% ION EQUATIONS
% COMPARTMENT Matrix:
ii = [1  2  3  4  5  6]; % Indices of SVs in compartment Matrix
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_Matrix./Km(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_Matrix./Kh(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_Matrix./Kh(ii)+m_Matrix./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 + 1*J_SCS_Matrix) / VWater_Matrix;
J_M = (0) / VWater_Matrix;
J_K = (0) / VWater_Matrix;
Phi_H = J_H - sum( h_Matrix*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_Matrix*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_Matrix*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_Matrix/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(7,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(8,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(9,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_SCS_Matrix];
